s = Server.default;

s.options.numAnalogInChannels = 8;
s.options.numAnalogOutChannels = 8;
s.options.numDigitalChannels = 16;

s.options.blockSize = 16;
s.options.numInputBusChannels = 2;
s.options.numOutputBusChannels = 2;

s.options.postln;
SerialPort.listDevices;

s.waitForBoot({

	SynthDef(\entangle_xm, {
		arg freq1 = 200,
			freq2 = 1,
			pan = 0,
			delay_time = 0.2,
			feedback_amount = 0,
			vol = 1;

		var local, osc1, osc2;

		local = LocalIn.ar(2) * feedback_amount;

		osc1 = Pan2.ar(SinOsc.ar(freq1.lag), pan.lag) + local;
		osc2 = SinOsc.ar(freq2.lag, mul: osc1);

		local = DelayC.ar(osc2, 8, delay_time.lag);
		local = local * feedback_amount.lag;

		LocalOut.ar(local.reverse);
		Out.ar(0, (osc2 * AnalogIn.ar(0).lag).sin * 0.318);

	}).add;

	p = SerialPort(
		"/dev/ttyUSB0",
		baudrate: 9600,
		crtscts: true);

	s.sync;

	x = Synth(\entangle_xm);

	r= Routine({
		var byte, str, res;
		99999.do{|i|
			if(p.read==10, {
				str = "";
				
				while({byte = p.read; byte !=13 }, {
					str= str++byte.asAscii;
				});

				res = str.split($ );
				
				~pot1 = res[0].asFloat / 1023.0;
				~pot2 = res[1].asFloat / 1023.0;
				~pot3 = res[2].asFloat / 1023.0;
				~slider = res[3].asFloat / 1023.0;
				~rot_enc = res[4].asFloat;
				~sw = res[5].asInt;

				if (~reset != 1, {
					~feedback = ~rot_enc.abs * 0.1;

					if (~prev_feedback != ~feedback, {
						x.set(\feedback_amount, ~feedback);
						~feedback.postln;
					});
					
					~prev_feedback = ~feedback;

					if (~pot1 > 0.55, {
						~pot1 = ~pot1.lincurve(0.55, 1, 200, 17000, 4);
					}, {
						if (~pot1 < 0.45, {
							~pot1 = ~pot1.lincurve(0, 0.45, 0, 200, 4);
						}, {
							~pot1 = 200;
						});
					});

					x.set(\freq1, ~pot1);

					if (~pot2 > 0.55, {
						~pot2 = ~pot2.lincurve(0.55, 1, 200, 17000, 4);
					}, {
						if (~pot2 < 0.45, {
							~pot2 = ~pot2.lincurve(0, 0.45, 0, 200, 4);
						}, {
							~pot2 = 200;
						});
					});

					x.set(\freq2, ~pot2);
					x.set(\delay_time, ~pot3 * 8);
					x.set(\pan, (~slider * 2) - 1);
				});
			});

			if (~sw == 1, {
				~feedback = 0;
				~reset = 1;
				x.set(\feedback_amount, ~feedback);
			}, {
				~reset = 0;
			});


		};
	}).play;

});